# File: RemoteFileIndex.py
# 
# Copyright (c) 2006 by Zindep
#
# GNU General Public Licence (GPL) see LICENSE.txt

__author__  = '''Nicolas Laurance <nlaurance@zindep.com>'''
__docformat__ = 'plaintext'


from AccessControl import ClassSecurityInfo
try:
    from Products.CMFCore.permissions import View, ModifyPortalContent
except:
    #plone 2.0.5
    from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

from Products.Archetypes.atapi import *

from Products.ATContentTypes.config import ICONMAP

from Products.RemoteFileIndex.config import *
##code-section module-header #fill in your manual code here
import urllib, os
from Products.CMFCore.utils import getToolByName
##/code-section module-header

schema=Schema((
    StringField('RemoteUrl',
        widget=StringWidget(
            label='URL',
            label_msgid='RemoteFileIndex_label_RemoteUrl',
            size=90,
            description_msgid='RemoteFileIndex_help_RemoteUrl',
            i18n_domain='RemoteFileIndex',
        )
    ),

),
)

RemoteFileIndex_schema = BaseSchema + \
    schema


class RemoteFileIndex(BaseContent):
    """A file on a remote server whose content is indexed in the portal catalog"""
    security = ClassSecurityInfo()
    __implements__ = (getattr(BaseContent,'__implements__',()),)

    # To avoid too many remote connections
    remote_size = 0
    remote_content_type = ''
    remote_modification_date = ''
    remote_filename = ''

    # This name appears in the 'add' box
    archetype_name             = 'RemoteFileIndex'

    meta_type                  = 'RemoteFileIndex'
    portal_type                = 'RemoteFileIndex'
    allowed_content_types      = []
    filter_content_types       = 0
    global_allow               = 1
    allow_discussion           = 0
    content_icon               = 'RemoteFileIndex.gif'
    immediate_view             = 'RemoteFileIndex_view'
    default_view               = 'RemoteFileIndex_view'
    suppl_views                = ()
    typeDescription            = "A file on a remote server whose content is indexed in the portal catalog"
    typeDescMsgId              = 'description_edit_remotefileindex'

    schema = RemoteFileIndex_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header
    actions=(
        {'action' : '''string:$object_url/RemoteFileIndex_view''',
         'category' : '''object''',
         'id' : 'view',
         'name' : 'View',
         'permissions' : ('''View''',),
         'condition' : 'python:1'},
        {'action' : '''string:$object_url/Refresh''',
         'category' : '''object''',
         'id' : 'refresh',
         'name' : 'Refresh',
         'permissions' : ('''Modify portal content''',),
         'condition' : ''},
         )

    #Methods


    security.declareProtected(ModifyPortalContent,'Refresh')
    def Refresh(self):
        """
        refreshes url and remote file properties
        """
        self.setRemoteUrl(self.getRemoteUrl())
        self.reindexObject()
        REQ=self.REQUEST
        REQ.RESPONSE.redirect('%s?portal_status_message=%s' % (self.absolute_url(),'index refreshed'))

    security.declareProtected(ModifyPortalContent,'setRemoteUrl')
    def setRemoteUrl(self, value, **kwargs):
        """
        updates url and remote file properties
        """
        self.Schema()['RemoteUrl'].set(self, value, **kwargs) # default set
        # socket connection
        try:
            sock = urllib.urlopen(self.RemoteUrl)
            infos=sock.info()
            self.remote_content_type = infos.gettype()
            self.remote_size = int(infos.getheader('Content-Length'))
            self.remote_filename = os.path.split(self.RemoteUrl)[1]
            self.remote_modification_date = infos.getheader('Last-Modified')
            sock.close()
        except :
            pass

    security.declareProtected(View, 'get_size')
    def get_size(self):
        """ZMI / Plone get size method
        """
        sz=self.remote_size
        if sz:
            return sz
        else:
            return 0

    security.declarePublic('getIcon')
    def getIcon(self, relative_to_portal=0):
        """Calculate the icon using the mime type of the file
        """
        field = self.getField('RemoteUrl')
        if not field or not self.get_size():
            # field is empty
            return BaseContent.getIcon(self, relative_to_portal)

        contenttype       = self.remote_content_type
        contenttype_major = contenttype and contenttype.split('/')[0] or ''

        mtr   = getToolByName(self, 'mimetypes_registry', None)
        utool = getToolByName( self, 'portal_url' )

        if ICONMAP.has_key(contenttype):
            icon = urllib.quote(ICONMAP[contenttype])
        elif ICONMAP.has_key(contenttype_major):
            icon = urllib.quote(ICONMAP[contenttype_major])
        else:
            mimetypeitem = mtr.lookup(contenttype)
            if not mimetypeitem:
                return BaseContent.getIcon(self, relative_to_portal)
            icon = mimetypeitem[0].icon_path

        if relative_to_portal:
            return icon
        else:
            # Relative to REQUEST['BASEPATH1']
            res = utool(relative=1) + '/' + icon
            while res[:1] == '/':
                res = res[1:]
            return res

    security.declareProtected(View, 'icon')
    def icon(self):
        """for ZMI
        """
        return self.getIcon()

    security.declarePrivate('txng_get')
    def txng_get(self, attr=('SearchableText',)):
        """
        Used by TextIndexNG :
        * Get File via urllib
        * Special searchable text source for text index ng 2
        taken from ATContentTypes/content/file.py
        """
        
        if attr[0] != 'SearchableText':
            # only a hook for searchable text
            return

        # Get File via urllib
        #import pdb; pdb.set_trace()
        source_url = self.RemoteUrl
        sock = urllib.urlopen(source_url)
        f  = sock.read()
        mt = sock.info().gettype()
        sock.close()
        
        #  Special searchable text source for text index ng 2
        source   = ''
        mimetype = 'text/plain'
        encoding = 'utf-8'

        # stage 1: get the searchable text and convert it to utf8
        sp    = getToolByName(self, 'portal_properties').site_properties
        stEnc = getattr(sp, 'default_charset', 'utf-8')
        st    = self.SearchableText()
        source+=unicode(st, stEnc).encode('utf-8')

        # get the file and try to convert it to utf8 text
        ptTool = getToolByName(self, 'portal_transforms')
        if f:
            try:
                result = ptTool.convertTo('text/plain', str(f), mimetype=mt)
                if result:
                    data = result.getData()
                else:
                    data = ''
            except TransformException:
                data = ''
            source+=data
            
        return source, mimetype, encoding

registerType(RemoteFileIndex,PROJECTNAME)
# end of class RemoteFileIndex
